import os
import threading
import time
import socket
import uuid
import shutil
import requests
from urllib import parse, request
from PyQt5.QtCore import QSettings
from kylindbus.enums import LOG_PATH, KYLIN_SOFTWARE_PROPERTIES_LOG, gZipFile, PINGBACK_INTERNET_FILE_URL, \
    PINGBACK_INTRANET_FILE_URL, \
    PINGBACK_INTRANET_URL, PINGBACK_INTERNET_URL
from kylindbus.log_manager import logger


class ConnectServerThread(threading.Thread):
    def __init__(self, dbusDaemon):
        threading.Thread.__init__(self)
        self.dbusDaemon = dbusDaemon
        self.appname = ""
        self.appversion = ""
        self.state = ""
        self.errorcode = ""

    def get_values(self, _appname="", _appversion="", _state='', _errorcode=''):
        self.appname = _appname
        self.appversion = _appversion
        self.state = _state
        self.errorcode = _errorcode

    def run(self):
        host_ip, version, soft_version, host_mac = '', '', '', ''
        # 获取本机ip
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            host_ip = s.getsockname()[0]
        except:
            host_ip = 'Ip failed to get'
        # 获取系统版本
        with open('/etc/lsb-release', 'r') as f:
            for line in f.readlines():
                if line.strip().startswith('DISTRIB_DESCRIPTION='):
                    versions = line.strip().split('=')
                    if "V10" in line and "SP1" in versions[1]:
                        version = "V10SP1"
                    else:
                        version = "V10Pro"
                    break
        # 获取软件版本
        output = os.popen('dpkg -l|grep kylin-update-manager').readlines()
        if output:
            soft_version = output[0].strip().split()[2]
        # 获取时间
        nowtime = time.strftime('%Y-%m-%d-%H:%M:%S', time.localtime(time.time()))
        # 获取Mac
        mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
        host_mac = ":".join([mac[e:e + 2] for e in range(0, 11, 2)])
        # 获取序列号
        if os.path.exists("/etc/.kyinfo"):
            settings = QSettings("/etc/.kyinfo", QSettings.IniFormat)
            settings.beginGroup("servicekey")
            key = str(settings.value("key")).strip()
            settings.endGroup()
        else:
            key = "0"
        try:
            # 用于收集源管理器的更新日志
            if self.errorcode == "10000049" or self.errorcode == "10000005":
                log_dir = os.path.join(LOG_PATH, host_mac + "_" + nowtime)
                log_file_gzip = log_dir + ".tar.gz"
                os.makedirs(log_dir, exist_ok=True)
                if os.path.exists(KYLIN_SOFTWARE_PROPERTIES_LOG):
                    with open(os.path.join(log_dir, "apt_list"), "w") as f:
                        for file in os.listdir("/var/lib/apt/lists"):
                            f.write(file + "\n")
                    shutil.copy(KYLIN_SOFTWARE_PROPERTIES_LOG, log_dir)
                gZipFile(log_dir, log_file_gzip)
                header = {'Content-Type': "multipart/form-data", "Accept-Encoding": "gzip"}
                try:
                    with open(log_file_gzip, "rb") as f:
                        requests.post(PINGBACK_INTRANET_FILE_URL + "filename=" + os.path.basename(log_file_gzip),
                                      data=f.read(), headers=header)
                except:
                    with open(log_file_gzip, "rb") as f:
                        requests.post(PINGBACK_INTERNET_FILE_URL + "filename=" + os.path.basename(log_file_gzip),
                                      data=f.read(), headers=header)
                shutil.rmtree(log_dir)
                os.remove(log_file_gzip)
            else:
                log_file_gzip = ""
            kmg = parse.urlencode(
                {'ip': host_ip, 'version': version, 'soft_version': soft_version, 'datetime': nowtime,
                 'host_mac': host_mac, 'appname': self.appname, 'appversion': self.appversion, 'serial_number': key,
                 'status': self.state, 'filename': log_file_gzip, 'errorcode': self.errorcode})
            # data = {}
            # data['ip'] = host_ip
            # data['vesion'] = version
            # data['soft_version'] = soft_version
            # data['datetime'] = nowtime
            # data['host_mac'] = host_mac
            # data['appname'] = self.appname
            # data['appversion'] = self.appversion
            # data['serial_number'] = key
            # data['status'] = self.state
            # data['filename'] = log_file_gzip
            # data['errorcode'] = self.errorcode
            # data = parse.urlencode(data).encode('utf-8')
            # request.urlopen(url=PINGBACK_INTRANET_URL, data=data, timeout=3)
            # 优先使用内网服务器，再使用外网
            try:
                url = PINGBACK_INTRANET_URL + kmg
                req = request.urlopen(url=url, timeout=3)
                logger.info("成功访问局域网日志服务器")
            except:
                url = PINGBACK_INTERNET_URL + kmg
                req = request.urlopen(url=url, timeout=3)
                logger.info("成功访问外网日志服务器")
        except Exception as e:
            logger.error("访问日志服务器失败")
