#!/usr/bin/python3

import json, yaml
from kylindbus.enums import JSON_PATH, JSON_PKGNAME
from kylindbus.log_manager import logger
import apt_pkg
import os

class ParseJson(object):
    def __init__(self, parent):
        # super(ParseJson, self).__init__(parent)
        self.parent = parent
        self.parent.aptHandler.init_cache()
        self.cache = parent.aptHandler.cache
        # list = self.get_all_upgrade_list()
        # print(list)

    def update_json(self):
        self.parent.init_cache()
        self.cache = self.parent.aptHandler.cache
        try:
            pkg = self.cache[JSON_PKGNAME]
            if not pkg.is_installed:
                pkg.mark_install()
                self.cache.commit()
                return True
            elif pkg.is_upgradable:
                pkg.mark_upgrade()
                self.cache.commit()
                return True
            else:
                return False
        except Exception as e:
            logger.info("更新json文件错误:",e)
            return False

    def read_multiple_json(self):
        json_file_list = []
        try:
            file = os.walk(JSON_PATH)
        except:
            return json_file_list
        for path,dir_list,file_list in file:
            for file_name in file_list:
                if file_name.endswith(".json"):
                    json_file_list.append(os.path.join(path, file_name))
        return json_file_list

    def read_json_and_yaml(self, appname, type):
        try:
            file = os.walk(JSON_PATH)
        except:
            return ""
        if type == 0:
            filetype = "json"
        elif type == 1:
            filetype = "yaml"
        else:
            filetype = ""
        for path,dir_list,file_list in file:
            for file_name in file_list:
                if file_name == (str(appname) + "." + filetype):
                    return os.path.join(path, file_name)
        return ""

    def get_virtual_packages_list(self):
        vir_pkg_list = []
        try:
            json_file_list = self.read_multiple_json()
        except:
            return  vir_pkg_list
        for json_file in json_file_list:
            with open(json_file, 'r') as f:
                data = json.load(f)
            pkgname = data['package']
            vir_pkg_list.append(pkgname)
        return vir_pkg_list

    def check_installed_or_upgradable(self, pkglist, check_installed=True):
        result_list = list()
        # 检测当前系统是否是efi安装
        if "efi" in os.popen("archdetect").readline().strip():
            efi_mode = True
        else:
            efi_mode = False
        # 获取静默安装列表
        for name in pkglist:
            if efi_mode and "grub-pc" in name:  # efi安装的系统grub只会使用grub-efi-*包，不会安装grub-pc，需要做区分
                logger.info("过滤:" + str(name))
                continue
            elif not efi_mode and "grub-efi" in name:
                logger.info("过滤:"+str(name))
                continue
            try:
                iter = self.cache[name]
                if check_installed:
                    if not iter.is_installed:
                        result_list.append(name)
                else:
                    if iter.is_upgradable:
                        result_list.append(name)
            except:
                continue
        return result_list

    def get_pkg_upgrade_info(self, name):
        pkg_info_dict = {}
        json_file = self.read_json_and_yaml(name, 0)
        yaml_file = self.read_json_and_yaml(name, 1)
        if not os.path.exists(json_file): #不存在json文件直接返回空字典
            pkg_info_dict["json"] = 0
            return pkg_info_dict
        try:
            with open(json_file, 'r') as json_file:
                data = json.load(json_file)
                try:
                    with open(yaml_file) as yaml_file:
                        yaml_data = yaml.load(yaml_file, Loader=yaml.FullLoader)
                        pkg_info_dict["changelog"] = yaml_data['changelog']
                except Exception as e:
                    pkg_info_dict["changelog"] = ""
                # source_version = str(pkg.versions[0]).split("=")[1]
                apt_pkg.init_system()
                '''if apt_pkg.version_compare(source_version, data['version']) < 0: #json文件中version小于等于源中版本，直接返回字典
                    pkg_info_dict['json'] = 1
                    return pkg_info_dict'''
                # 获取包名、版本、中文名、描述等信息
                pkg_info_dict['package'] = data['package']
                pkg_info_dict['version'] = data['version']
                if (os.getenv('LANG') == "zh_CN.UTF-8"):
                    pkg_info_dict['name'] = data['name']['zh_CN']
                    pkg_info_dict['description'] = data['description']['zh_CN']
                else:
                    pkg_info_dict['name'] = data['name']['en_US']
                    pkg_info_dict['description'] = data['description']['en_US']
                pkg_info_dict['icon'] = data['icon']
                # 获取静默安装列表
                pkg_info_dict['silent_install_list'] = self.check_installed_or_upgradable(data['silent_install_list'])
                # 获取静默升级列表
                pkg_info_dict['silent_upgrade_list'] = self.check_installed_or_upgradable(data['silent_upgrade_list'], False)
                # 获取安装列表
                pkg_info_dict['install_list'] = self.check_installed_or_upgradable(data['install_list'])
                # 获取升级列表
                pkg_info_dict['upgrade_list'] = self.check_installed_or_upgradable(data['upgrade_list'], False)
                pkg_info_dict['json'] = 2
                return pkg_info_dict
        except Exception as e:
            logger.info("json文件不存在或者内容出错:", e)
            pkg_info_dict["json"] = 0
            return pkg_info_dict

if __name__ == "__main__":
    parse_json = ParseJson()
