#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os
import re
import shutil
import sqlite3
from kylindbus.enums import DB_FILE, INSTALLED_LIST, DISPALY_LIST
from kylindbus.log_manager import logger


class Sqlite3Server(object):
    def __init__(self):
        self.init_sqlit()

    # 初始化连接数据库
    def init_sqlit(self):
        logger.info("初始化连接数据库")
        if os.path.isfile(DB_FILE):
            self.connect = sqlite3.connect(DB_FILE, check_same_thread=False)
            self.cursor = self.connect.cursor()
            self.insert_new_field()
        else:
            if not os.path.isdir(os.path.dirname(DB_FILE)):
                os.makedirs(os.path.dirname(DB_FILE))
            shutil.copy("/usr/share/kylin-update-manager/kylin-update-manager.db", os.path.dirname(DB_FILE))
            self.connect = sqlite3.connect(DB_FILE, check_same_thread=False)
            self.cursor = self.connect.cursor()
            self.insert_new_field()
        logger.info("成功连接数据库")

    # 数据库表格中动态增加新的字段用于扩展
    def insert_new_field(self):
        if len(INSTALLED_LIST) == 0 and len(DISPALY_LIST) == 0:
            return
        self.cursor.execute("select sql from sqlite_master where name='installed'")
        installed_sql = self.cursor.fetchone()[0]
        pattern = re.compile(r'\"\w+\"')
        installed_sql_list = pattern.findall(installed_sql)
        for value in INSTALLED_LIST:
            for field in installed_sql_list:
                if value["item"] == str(field).strip("\""):
                    break
                elif field == installed_sql_list[len(installed_sql_list) - 1]:
                    try:
                        if value["default"] != "":
                            sql = 'alter table installed add column "' + value["item"] + '" ' + value["type"] \
                                  + ' default ' + str(value["default"])
                        else:
                            sql = 'alter table installed add column "' + value["item"] + '" ' + value["type"]
                        self.cursor.execute(sql)
                        logger.info("installed表插入新字段:" + value["item"])
                    except:
                        logger.error("installed表插入新字段失败:" + value["item"], exc_info=True)

        self.cursor.execute("select sql from sqlite_master where name='display'")
        display_sql = self.cursor.fetchone()[0]
        pattern = re.compile(r'\"\w+\"')
        display_sql_list = pattern.findall(display_sql)
        for value in DISPALY_LIST:
            for field in display_sql_list:
                if value["item"] == str(field).strip("\""):
                    break
                elif field == display_sql_list[len(display_sql_list) - 1]:
                    try:
                        if value["default"] != "":
                            sql = 'alter table display add column "' + value["item"] + '" ' + value["type"] \
                                  + ' default ' + str(value["default"])
                        else:
                            sql = 'alter table installed add column "' + value["item"] + '" ' + value["type"]
                        self.cursor.execute(sql)
                        logger.info("display表插入新字段：" + value["item"])
                    except:
                        logger.error("display表插入新字段失败：" + value["item"], exc_info=True)

    # 写入数据到installed表中
    def insert_into_installed(self, *args, **kwargs):
        self.cursor.execute(
            "insert into installed (appname, version, time, description, icon, statue, keyword, errorcode) values(?,"
            "?,?,?,?,?,?,?)",
            (args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7]))
        self.connect.commit()

    # 写入数据到display表中
    def insert_into_display(self, *args, **kwargs):
        sql = "update display set " + args[0] + "='" + args[1] + "' where id = 1"
        self.cursor.execute(sql)
        self.connect.commit()
